/**
 * @file FileHandle.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "FileHandle.h"
#include <vector>

CFileHandle::~CFileHandle()
{
	Close();
}

bool CFileHandle::Open(LPCWSTR FileName, bool Create)
{
	h = CreateFile(FileName, Create ? GENERIC_WRITE : GENERIC_READ, FILE_SHARE_READ, NULL, Create ? CREATE_ALWAYS : OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	return h != INVALID_HANDLE_VALUE;
}

bool CFileHandle::Read(void* Data, DWORD Count, DWORD *Read/*= NULL*/)
{
	DWORD nRead;
	if (h != INVALID_HANDLE_VALUE)
	{
		if (ReadFile(h, Data, Count, &nRead, NULL))
		{
			if (!Read)
				return Count == nRead;
			*Read = nRead;
			return true;
		}
	}
	return false;
}

bool CFileHandle::Write(void* Data, DWORD Count)
{
	DWORD nWritten;
	if (h != INVALID_HANDLE_VALUE)
		if (WriteFile(h, Data, Count, &nWritten, 0))
			if (Count == nWritten)
				return true;
	return false;
}

bool CFileHandle::Seek(int Offset, DWORD Direction)
{
	if (h != INVALID_HANDLE_VALUE)
	{
		DWORD newPos = SetFilePointer(h, Offset, NULL, Direction);
		if (newPos != INVALID_SET_FILE_POINTER)
			return true;
	}
	return false;
}

void CFileHandle::Close()
{
	if (h != INVALID_HANDLE_VALUE)
	{
		CloseHandle(h);
		h = INVALID_HANDLE_VALUE;
	}
}

//------- CTextFile

bool CTextFile::WriteLN()
{
	return WriteText(L"\r\n");
}

bool CTextFile::WriteText(const std::wstring &Text)
{
	size_t len = Text.size();
	if (len > 0)
	{
		std::vector<char> buf;
		int size = WideCharToMultiByte(CodePage, 0, &Text[0], (int) len, NULL, 0, NULL, NULL);
		if (size == 0)
			return false;
		buf.resize(size);
		if (!WideCharToMultiByte(CodePage, 0, &Text[0], (int) len, &buf[0], size, NULL, NULL))
			return false;
		return Write(&buf[0], size);
	}
	return true;
}

bool CTextFile::WriteLine(const std::wstring &Line)
{
	return WriteText(Line) && WriteLN();
}

