/**
 * @file Command.h
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#pragma once

#include "autocomplete/Autocomplete.h"
#include "autocomplete/ParamValidation.h"
#include "output_format/OutputHandler.h"
#include "resource.h"

#define GENERATE_HELP_ID_METHODS(cmd) \
	virtual unsigned int GetHelpFullId() override { return IDS_HELP_##cmd##; }\
	virtual unsigned int GetHelpShortId() override { return IDS_HELP_##cmd##_SHORT; }\
	virtual unsigned int GetHelpExampleId() override { return IDS_HELP_##cmd##_EXAMPLE; }

class Command
{
public:
	virtual ~Command() {};

	enum SyntaxType
	{
		mandatory,
		optional,
		conditional, // mandatory if previous is present, otherwise should be missing. this can not follow mandatory parameter!
	};

	virtual void Execute(elementArray &elements, OutputHandler &out) = 0;

	Command *AddKeyword(const std::wstring &text, Command::SyntaxType type, int skipArgs = 0);
	Command *AddParam(unsigned int argNum, Command::SyntaxType type, unsigned int helpId, std::shared_ptr<ParamValidator> validator);
	void GrabArguments(elementArray &elements, const std::wstring &commandString, AutocompleteCollector *collector);

	std::shared_ptr<Element> GetArgument(unsigned int argNum);
	std::wstring GetArgumentAsString(unsigned int argNum);

	virtual unsigned int GetHelpFullId() = 0;
	virtual unsigned int GetHelpShortId() = 0;
	virtual unsigned int GetHelpExampleId() = 0;


	struct CommandSyntax
	{
		std::wstring text;
		unsigned int argNum;
		unsigned int helpId;
		SyntaxType syntaxType;
		int autoCompleteSkipArgs; // how many arguments at the begining of the command have fixed order (usually 0)
		std::shared_ptr<ParamValidator> validator;
	};
	const std::vector<CommandSyntax>& GetSyntax() { return syntax; }

protected:
	Command *AddSyntax(const std::wstring &text, unsigned int argNum, Command::SyntaxType syntaxType, unsigned int helpId, std::shared_ptr<ParamValidator> validator, int skipArgs);
	elementArray arguments;
	std::vector<CommandSyntax> syntax;
};

