/**
 * @file ConnectionParams.h
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#pragma once

#define CONN_PAR_PREFIX L"--"
#define CONN_PAR_SERVER				L"s"
#define CONN_PAR_USERNAME			L"u"
#define CONN_PAR_USERNAME_DOMAIN	L"ud"
#define CONN_PAR_USERNAME_CURRENT	L"uc"
#define CONN_PAR_PASSWORD			L"p"
#define CONN_PAR_PASSWORD_ASK		L"pa"
#define CONN_PAR_ASK_ALL			L"aa"
#define CONN_PAR_ALTERNATIVE_STARTUP_SCRIPT L"startup"

#define CONN_DEFAULT_SERVER L"localhost"
#define CONN_DEFAULT_PORT 2226
#define CONN_DEFAULT_PASSWORD L""

struct ConnectionParams
{
	ConnectionParams();
	std::wstring host;
	uint16_t port;
	std::wstring user;
	std::wstring password;
	bool userDomain;
	bool userCurrent;
	bool askPassword;
	bool askAll;

	std::wstring alternativeStartupScript;

#ifdef _DEBUG
	friend std::wostream& operator<<(std::wostream &os, const ConnectionParams &cp)
	{
		return os << L"host: " << cp.host << std::endl
			<< L"port: " << cp.port << std::endl
			<< L"user: " << cp.user << std::endl
			<< L"password: " << cp.password << std::endl
			<< L"userDomain: " << cp.userDomain << std::endl
			<< L"userCurrent: " << cp.userCurrent << std::endl
			<< L"askPassword: " << cp.askPassword << std::endl
			<< L"askAll: " << cp.askAll << std::endl
			<< L"alternativeStartupScript: " << cp.alternativeStartupScript << std::endl;
	}
#endif
};

class ConnectionParamsProcessor
{
public:
	ConnectionParamsProcessor(const tokenArray &tokens)
		: tokens(tokens)
	{}

	std::pair<tokenArray::const_iterator, tokenArray::const_iterator> ProcessTo(ConnectionParams &connectionParams);
private:
	static bool IsConnectionParam(const std::wstring &currTokenText);
	const std::wstring &GetConnectionArgument(tokenArray::const_iterator tokensIt);
	void GetUser(tokenArray::const_iterator tokensIt, ConnectionParams &connectionParams);

	const tokenArray &tokens;
};
