/**
 * @file ConsoleHistory.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "ConsoleHistory.h"

// shell command history
ConsoleHistory gCmdHistory;


#define HISTORY_SIZE 20

#define UTF16_LE std::codecvt_utf16<wchar_t, 0x10ffff, std::little_endian>

void ConsoleHistory::Add(std::wstring Line)
{
	if (Line.empty())
		return;

	// handling duplicity
	auto pos = std::find(Items.begin(), Items.end(), Line);
	if (pos != Items.end())
	{
		Items.erase(pos);
	}

	if (Items.size() >= HISTORY_SIZE)
	{
		Items.erase(Items.begin());
	}

	Items.push_back(Line);

	GoEnd();
}

void ConsoleHistory::Load()
{
	std::wifstream in;
	std::wstring line;
	in.open(FileName, std::ofstream::binary);
	in.imbue(std::locale(in.getloc(), new UTF16_LE));
	bool begin = true;
	while (std::getline(in, line))
	{
		if (line.empty())
			continue;
		if (begin)
		{
			if (line[0] == 0x0FEFF) // handling BOM
				line.erase(0, 1);
			begin = false;
		}
		while (line.back() == L'\r' || line.back() == L'\n')
			line.resize(line.size()-1);
		Add(line.c_str());
	}
}

void ConsoleHistory::Save()
{
	std::wofstream out;
	out.open(FileName, std::ofstream::binary);
	out.imbue(std::locale(out.getloc(), new UTF16_LE));
	for (size_t i = 0; i < Items.size(); ++i)
	{
		std::wstring &line = Items[i];
		out << line << L"\r\n";
	}
	out.close();
}

size_t ConsoleHistory::GetCount() const
{
	return Items.size();
}

void ConsoleHistory::Clear()
{
	Items.clear();
	GoEnd();
}

void ConsoleHistory::GoBegin()
{
	CurrentIndex = -1;
}

void ConsoleHistory::GoEnd()
{
	CurrentIndex = Items.size();
}

std::wstring ConsoleHistory::GetNextItem(bool Prior /* = false */)
{
	if (Items.size() == 0)
		return L"";

	if (Prior)
	{
		if (CurrentIndex > 0)
			CurrentIndex--;
		return Items[CurrentIndex];
	}
	else
	{
		if (CurrentIndex + 1 < Items.size())
			CurrentIndex++;
		if (CurrentIndex >= Items.size())
			return L"";
		return Items[CurrentIndex];
	}
}

