/**
 * @file TableReader.h
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#pragma once

#include <Common/era_api/era_api.h>

class TableReader
{

public:
	virtual ~TableReader() {};

public:
	virtual StringVector GetColumns_keyword() = 0;
	virtual StringVector GetColumns_pretty() = 0;
	virtual bool HasNext() = 0;
	virtual StringVector GetNext() = 0;
};

class TableReader_Const : public TableReader
{

public:
	TableReader_Const(const StringVector &columnNames, const StringVector &prettyColumnNames, const std::vector<StringVector> &rows);

public:
	virtual StringVector GetColumns_keyword() override;
	virtual StringVector GetColumns_pretty() override;
	virtual bool HasNext() override;
	virtual StringVector GetNext() override;

private:

	const StringVector &columnNames;
	const StringVector &prettyColumnNames;
	const std::vector<StringVector> &rows;
	size_t position;
};

class TableReader_Api : public TableReader
{

public:
	TableReader_Api(const StringVector &columnNames, const StringVector &prettyColumnNames, Era_TableReader &apiTableReader, const bool prettyFields);

public:
	virtual StringVector GetColumns_keyword() override;
	virtual StringVector GetColumns_pretty() override;
	virtual bool HasNext() override;
	virtual StringVector GetNext() override;

private:
	void prepareNextRow();

private:

	const StringVector &columnNames;
	const StringVector &prettyColumnNames;
	Era_TableReader &apiTableReader;
	const bool prettyFields;

	StringVector nextRow;
	bool hasNextRow;
};
