/**
 * @file ParamValidation.h
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#pragma once

class Command;
class AutocompleteCollector;
class CommandList;


// BASE CLASS for other param validators
class ParamValidator
{
public:
	ParamValidator(bool allowList) : allowList(allowList) {}
	virtual void ValidateParam(std::shared_ptr<Element> param, Command *currentCommand);
	virtual void AutoComplete(std::shared_ptr<Element> param, Command *currentCommand, AutocompleteCollector *collector);
protected:
	virtual void ValidateAllowList(const std::vector<const ElementWord *> &words, std::shared_ptr<Element> param);
	virtual void ValidateSingleParam(const Element *param);
	void AutoCompleteWithList(const std::wstring &word, const std::vector<std::wstring> &options, AutocompleteCollector *collector);
	bool allowList;
	std::vector<std::wstring> enumValues;
};


// accepts all single params, but no list. makes no autocomplete
class ParamValidatorNoList : public ParamValidator
{
public:
	ParamValidatorNoList();
protected:
	virtual void ValidateSingleParam(const Element *param) override {} // accepts all
};


class ParamValidatorEnum : public ParamValidator
{
public:
	ParamValidatorEnum(std::wstring enumCommaList, bool allowList, bool validate = true);
	virtual void ValidateParam(std::shared_ptr<Element> param, Command *currentCommand) override;
protected:
	bool validate;
};


//requires existing parameter before offering/validating values
class ParamValidatorEnumConditional : public ParamValidatorEnum
{
public:
	ParamValidatorEnumConditional(int tableNameArgumentIndex, std::wstring enumCommaList, bool allowList, bool validate = true);
	virtual void ValidateParam(std::shared_ptr<Element> param, Command *currentCommand) override;	
	virtual void AutoComplete(std::shared_ptr<Element> param, Command *currentCommand, AutocompleteCollector *collector) override;
protected:
	int tableNameArgumentIndex;
	bool validate;
};


class ParamValidatorFlag : public ParamValidator
{
public:
	ParamValidatorFlag(bool allowList, bool addSave);
};

class ParamValidatorHelp : public ParamValidator
{
public:
	ParamValidatorHelp(const CommandList &commandList);
	virtual void ValidateParam(std::shared_ptr<Element> param, Command *currentCommand);
};


class ParamValidatorTables : public ParamValidator
{
public:
	ParamValidatorTables(bool allowList);
};


class ParamValidatorServerCfgItems : public ParamValidator
{
	bool OnlyPwd;
public:
	ParamValidatorServerCfgItems(bool allowList, bool onlyPwd);
	virtual void ValidateParam(std::shared_ptr<Element> param, Command *currentCommand);
};


// columns for table defined in other param
// this class makes no validation, only autocomplete
class ParamValidatorColumns : public ParamValidator
{
public:
	ParamValidatorColumns(int tableNameArgumentIndex, bool allowList, bool selectableOnly);
	virtual void ValidateParam(std::shared_ptr<Element> param, Command *currentCommand);
	virtual void AutoComplete(std::shared_ptr<Element> param, Command *currentCommand, AutocompleteCollector *collector) override;
protected:
	int tableNameArgumentIndex;
	bool selectableOnly;
};


// columns which are subset of previously defined columns, supporting also values on right side of operator
// this class makes no validation, only autocomplete
class ParamValidatorColumnsWithValues : public ParamValidator
{
public:
	ParamValidatorColumnsWithValues(int tableNameArgumentIndex, bool allowList);
	virtual void ValidateParam(std::shared_ptr<Element> param, Command *currentCommand) override;
	virtual void AutoComplete(std::shared_ptr<Element> param, Command *currentCommand, AutocompleteCollector *collector) override;
protected:
	int tableNameArgumentIndex;
};


