/**
 * @file Command_GroupCreate.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_GroupCreate.h"

#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"


void Command_GroupCreate::Execute(elementArray &elements, OutputHandler &out)
{
	Era_GroupParams *params = era_GroupParams_new();
	scope_exit se_query(std::bind(era_GroupParams_delete, params));

	FillGroupParams(0, *params);

	uint32_t newGroupId = -1;
	uint32_t result_groupCreate = era_Group_create(&newGroupId, params);
	if (result_groupCreate != 0)
	{
		throw EraCMDException_FromApi(result_groupCreate);
	}

	StringVector header(1);
	header[0] = L"group_id";
	StringVector prettyHeader(1);
	prettyHeader[0] = LoadStringFromResource(IDS_GROUP_ID);
	std::vector<StringVector> rows;
	StringVector data(1);
	data[0] = std::to_wstring((long long)newGroupId);
	rows.push_back(data);
	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

void Command_GroupCreate::FillGroupParams(const int argNumOffset, Era_GroupParams &params)
{
	Era_GroupType groupType;
	{
		const std::wstring elementStr = GetArgument(argNumOffset + 0)->GetString();

		if (_wcsicmp(elementStr.c_str(), L"static") == 0)
			groupType = ERA_GROUP_TYPE__STATIC;
		if (_wcsicmp(elementStr.c_str(), L"parametric") == 0)
			groupType = ERA_GROUP_TYPE__PARAMETRIC;

		era_GroupParams_SetType(&params, groupType);
	}

	{
		const std::wstring elementStr = GetArgument(argNumOffset + 1)->GetString();

		era_GroupParams_SetName(&params, elementStr.c_str());
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 2))
	{
		era_GroupParams_SetDescription(&params, element->GetString().c_str());
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 3))
	{	
		std::wstring parentIdStr = element->GetString();
		if (_wcsicmp(parentIdStr.c_str(), L"!nogroup") == 0)
		{
			era_GroupParams_SetNoParent(&params);
		}
		else
		{
			uint32_t num = 0;
			try
			{
				num = WstringToUint(parentIdStr);
			}
			catch (...)
			{
				throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_GROUP_ID, parentIdStr.c_str());
			}
			era_GroupParams_SetParentId(&params, num);
		}
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 4))
	{
		if (groupType == ERA_GROUP_TYPE__PARAMETRIC)
			era_GroupParams_SetParamsXml(&params, element->GetString().c_str());
		else
			throw EraCMDException(CMDERROR_GROUP_PARAM_NOT_APPLICABLE);
	}
	else
	{
		if (groupType == ERA_GROUP_TYPE__PARAMETRIC)
			throw EraCMDException(CMDERROR_GROUP_PARAM_MISSING);
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 5))
	{
		if (groupType == ERA_GROUP_TYPE__PARAMETRIC)
		{
			const std::wstring elementStr = element->GetString();

			if (_wcsicmp(elementStr.c_str(), L"TRUE") == 0)
				era_GroupParams_SetSticky(&params, ERA_GROUP_FLAG_STATE__SET);
			if (_wcsicmp(elementStr.c_str(), L"FALSE") == 0)
				era_GroupParams_SetSticky(&params, ERA_GROUP_FLAG_STATE__UNSET);
		}
		else
			throw EraCMDException(CMDERROR_GROUP_PARAM_NOT_APPLICABLE);
	}
	else
	{
		if (groupType == ERA_GROUP_TYPE__PARAMETRIC)
			era_GroupParams_SetSticky(&params, ERA_GROUP_FLAG_STATE__UNSET);
	}
}
