/**
 * @file Command_GroupUpdate.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_GroupUpdate.h"

#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

void Command_GroupUpdate::FillGroupParams(Era_GroupParams &params)
{
	bool bAtLeastOneParam = false;

	if (std::shared_ptr<Element> element = GetArgument(1))
	{
		era_GroupParams_SetName(&params, element->GetString().c_str());
		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(2))
	{
		era_GroupParams_SetDescription(&params, element->GetString().c_str());
		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(3))
	{
		std::wstring parentIdStr = element->GetString();
		if (_wcsicmp(parentIdStr.c_str(), L"!nogroup") == 0)
		{
			era_GroupParams_SetNoParent(&params);
		}
		else
		{
			uint32_t parentId;
			try
			{
				parentId = WstringToUint(parentIdStr);
			}
			catch (...)
			{
				throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_GROUP_ID, parentIdStr.c_str());
			}

			era_GroupParams_SetParentId(&params, parentId);
		}
		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(4))
	{
		era_GroupParams_SetParamsXml(&params, element->GetString().c_str());
		bAtLeastOneParam = true;
	}

	if (std::shared_ptr<Element> element = GetArgument(5))
	{
		const std::wstring elementStr = element->GetString();

		if (_wcsicmp(elementStr.c_str(), L"TRUE") == 0)
			era_GroupParams_SetSticky(&params, ERA_GROUP_FLAG_STATE__SET);
		if (_wcsicmp(elementStr.c_str(), L"FALSE") == 0)
			era_GroupParams_SetSticky(&params, ERA_GROUP_FLAG_STATE__UNSET);

		bAtLeastOneParam = true;
	}

	if (!bAtLeastOneParam)
		throw EraCMDException(CMDERROR_GROUP_UPDATE_MISSING_PARAMS);
}

void Command_GroupUpdate::Execute(elementArray &elements, OutputHandler &out)
{
	Era_GroupParams* params = era_GroupParams_new();
	scope_exit se_params(std::bind(era_GroupParams_delete, params));

	uint32_t updatedGroupId = 0;
	{
		const std::wstring elementStr = GetArgument(0)->GetString();

		try
		{
			updatedGroupId = ::WstringToUint(elementStr);
		}
		catch (...)
		{
			throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_GROUP_ID, elementStr.c_str());
		}
	}

	FillGroupParams(*params);

	uint32_t result_groupUpdate = era_Group_update(updatedGroupId, params);
	if (result_groupUpdate != 0)
	{
		throw EraCMDException_FromApi(result_groupUpdate);
	}
}
