/**
 * @file Command_License.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_License.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"

void Command_License::Execute(elementArray &elements, OutputHandler &out)
{
	Era_LicenseInfo *licenseInfo;


	uint32_t rc = era_GetLicenseInfo(&licenseInfo);
	if (rc != ERAERROR_SUCCESS)
	{
		throw EraCMDException_FromApi(rc);
	}
	scope_exit se_LicenseInfo(std::bind(era_LicenseInfo_delete, licenseInfo));

	StringVector header(2);
	header[0] = L"item";
	header[1] = L"value";

	StringVector prettyHeader(2);
	prettyHeader[0] = LoadStringFromResource(IDS_ITEM);
	prettyHeader[1] = LoadStringFromResource(IDS_VALUE);

	std::vector<StringVector> rows;

	StringVector row(2);
	row[0] = LoadStringFromResource(IDS_LICENSE_LICENSED_TO);
	row[1] = era_LicenseInfo_GetCustomerName(licenseInfo);
	rows.push_back(row);

	row[0] = LoadStringFromResource(IDS_LICENSE_NUMBER_OF_CLIENTS);
	row[1] = std::to_wstring((long long)era_LicenseInfo_GetTotalCount(licenseInfo));
	rows.push_back(row);

	int64_t time = era_LicenseInfo_GetExpirationTime(licenseInfo);
	struct tm expt;
	_gmtime64_s(&expt, &time);
	WCHAR expirationTime[100];
	swprintf_s(expirationTime, _countof(expirationTime), L"%04d-%02d-%02d", expt.tm_year + 1900, expt.tm_mon + 1, expt.tm_mday);
	std::wstring expirationLabel = era_LicenseInfo_IsEvalVersion(licenseInfo) ? LoadStringFromResource(IDS_LICENSE_NEVER_EXPIRES) : LoadStringFromResource(IDS_LICENSE_EXPIRES);
	row[0] = expirationLabel;
	row[1] = expirationTime;
	rows.push_back(row);

	row[0] = LoadStringFromResource(IDS_LICENSE_NUMBER_OF_KEYS);
	row[1] = std::to_wstring((long long)era_LicenseInfo_GetLicensesCount(licenseInfo));
	rows.push_back(row);

	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

