/**
 * @file Command_LogForward.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_LogForward.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"
#include "../Element.h"

Command_LogForward::Command_LogForward(CommandFlagsStack &commandFlagsStack)
	: commandFlagsStack(commandFlagsStack)
{
}

void Command_LogForward::Execute(elementArray &elements, OutputHandler &out)
{
	std::wstring logType;

	std::shared_ptr<Element> arg1 = GetArgument(0);
	std::shared_ptr<Element> arg2 = GetArgument(1);
	std::shared_ptr<Element> arg3 = GetArgument(2);
	std::shared_ptr<Element> arg4 = GetArgument(3);
	std::shared_ptr<Element> arg5 = GetArgument(4);

	if (arg1)
		logType = arg1->GetString();

	// detect list mode
	if (arg1 == NULL || arg2 == NULL)
	{
		bool pretty = commandFlagsStack.CurrentFlags().fieldFormatting == CommandFlags::FIELD_PRETTY;

		Era_LogForwardInfo* info;
		uint32_t rc = era_GetLogForwardInfo(logType.c_str(), &info, pretty);
		if (rc != ERAERROR_SUCCESS)
		{
			throw EraCMDException_FromApi(rc);
		}
		scope_exit se_LicenseInfo(std::bind(era_LogForwardInfo_delete, info));

		StringVector header(5);
		header[0] = L"log";
		header[1] = L"enabled";
		header[2] = L"severity";
		header[3] = L"facility";
		header[4] = L"level";

		StringVector prettyHeader(5);
		prettyHeader[0] = LoadStringFromResource(IDS_FORWARD_LOG);
		prettyHeader[1] = LoadStringFromResource(IDS_FORWARD_ENABLED);
		prettyHeader[2] = LoadStringFromResource(IDS_FORWARD_SEVERITY);
		prettyHeader[3] = LoadStringFromResource(IDS_FORWARD_FACILITY);
		prettyHeader[4] = LoadStringFromResource(IDS_FORWARD_LEVEL);

		std::vector<StringVector> rows;
		uint32_t count = era_LogForwardInfo_GetLogForwardsCount(info);
		StringVector row(5);
		for (uint32_t i = 0; i < count; i++)
		{
			const Era_LogForward *item = era_LogForwardInfo_GetLogForward(info, i);
			row[0] = era_LogForward_GetLog(item);
			row[1] = era_LogForward_GetEnabled(item);
			row[2] = era_LogForward_GetSeverity(item);
			row[3] = era_LogForward_GetFacility(item);
			row[4] = era_LogForward_GetLevel(item);
			rows.push_back(row);
		}

		TableReader_Const reader(header, prettyHeader, rows);
		out.Print(reader);

	}
	else
	{
		// mandatory parameter
		if (arg2 == NULL)
			throw EraCMDException(CMDERROR_COMMAND_MISSING_ARGUMENT, L"enable");

		bool enable = _wcsicmp(arg2->GetString().c_str(), L"true") == 0;

		// optional parameters
		std::wstring level, severity, facility;

		if (arg3 != NULL)
			level = arg3->GetString();

		if (arg4 != NULL)
			severity = arg4->GetString();

		if (arg5 != NULL)
			facility = arg5->GetString();

		uint32_t rc = era_SetLogForward(logType.c_str(), enable, arg3 ? level.c_str() : NULL, arg4 ? severity.c_str() : NULL, arg5 ? facility.c_str() : NULL);
		if (rc != ERAERROR_SUCCESS)
		{
			throw EraCMDException_FromApi(rc);
		}
	}
}

