/**
 * @file Command_ReportGenerate.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_ReportGenerate.h"

#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"
#include "../Element.h"

void Command_ReportGenerate::Execute(elementArray &elements, OutputHandler &out)
{
	uint32_t reportId = 0;
	{
		const std::wstring elementStr = GetArgument(0)->GetString();

		try
		{
			reportId = ::WstringToUint(elementStr);
		}
		catch (...)
		{
			throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_TEMPLATE_ID, elementStr.c_str());
		}
	}

	std::wstring outputDirectoryPath = GetArgument(1)->GetString();

	Era_ReportOutput outputType = ERA_REPORT_OUTPUT__HTML;
	if (std::shared_ptr<Element> element = GetArgument(2))
	{
		if (::_wcsicmp(element->GetString().c_str(), L"html") == 0)
			outputType = ERA_REPORT_OUTPUT__HTML;
		if (::_wcsicmp(element->GetString().c_str(), L"zip") == 0)
			outputType = ERA_REPORT_OUTPUT__ZIP;
		if (::_wcsicmp(element->GetString().c_str(), L"pdf") == 0)
			outputType = ERA_REPORT_OUTPUT__PDF;
	}

	uint32_t result = era_GenerateStaticReport(reportId, outputDirectoryPath.c_str(), outputType);
	if (result != 0)
		throw EraCMDException_FromApi(result);
}

