/**
 * @file Command_ReportUpdate.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_ReportUpdate.h"

#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

void Command_ReportUpdate::Execute(elementArray &elements, OutputHandler &out)
{
	Era_ReportTemplateParams *params = era_ReportTemplateParams_new();
	scope_exit se_query(std::bind(era_ReportTemplateParams_delete, params));

	uint32_t updatedReportId = 0;
	{
		const std::wstring elementStr = GetArgument(0)->GetString();

		try
		{
			updatedReportId = ::WstringToUint(elementStr);
		}
		catch (...)
		{
			throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_REPORT_ID, elementStr.c_str());
		}
	}

	bool bAtLeastOneParam = FillReportParams(1, *params, true);
	if (!bAtLeastOneParam)
		throw EraCMDException(CMDERROR_REPORT_TEMPLATE_UPDATE_MISSING_PARAMS);

	uint32_t result_reportUpdate = era_ReportTemplate_update(updatedReportId, params);
	if (result_reportUpdate != 0)
	{
		throw EraCMDException_FromApi(result_reportUpdate);
	}
}

