/**
 * @file Command_Set.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_Set.h"

#include <Common/era_api/err_codes.h>
#include <Common/FileHandle.h>
#include "../flags/CommandFlags.h"
#include "../flags/FlagsRegistry.h"
#include "../util/fileUtils.h"
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

extern FlagsRegistry gFlagsRegistry;

Command_Set::Command_Set(CommandFlags &currentFlags)
:
	currentFlags(currentFlags)
{
}

void Command_Set::Execute(elementArray &elements, OutputHandler &out)
{
	if (! GetArgument(0))
	{
		PrintFlagsState(gFlagsRegistry.GetSupportedFlags(), out);
		return;
	}

	if (::_wcsicmp(GetArgument(0)->GetString().c_str(), L"save") == 0)
	{
		std::wstring startupScriptFile;
		if (GetArgument(1))
		{
			startupScriptFile = GetArgument(1)->GetString();
		}
		else
		{
			startupScriptFile = GetProgramDataFilePath(L"eracmd_startup.txt");
		}

		SaveFlagsState(startupScriptFile);
		return;
	}

	if (! GetArgument(1))
	{
		FlagsRegistry::Flags flags;
		flags.push_back(gFlagsRegistry.GetFlag(GetArgument(0)->GetString()));

		PrintFlagsState(flags, out);
		return;
	}

	gFlagsRegistry.GetFlag(GetArgument(0)->GetString())->SetTo(currentFlags, GetArgument(1)->GetString());
}

void Command_Set::PrintFlagsState(const FlagsRegistry::Flags &flags, OutputHandler &out)
{
	StringVector header(3);
	header[0] = L"name";
	header[1] = L"current_value";
	header[2] = L"default_value";

	StringVector prettyHeader(3);
	prettyHeader[0] = LoadStringFromResource(IDS_FLAGS_NAME);
	prettyHeader[1] = LoadStringFromResource(IDS_FLAGS_CURRENT_VALUE);
	prettyHeader[2] = LoadStringFromResource(IDS_FLAGS_DEFAULT_VALUE);

	CommandFlags defaultFlags;

	std::vector<StringVector> rows;

	for (FlagsRegistry::Flags::const_iterator it = flags.begin(); it != flags.end(); ++it)
	{
		StringVector row(3);
		row[0] = (*it)->GetName();
		row[1] = (*it)->GetFrom(currentFlags);
		row[2] = (*it)->GetFrom(defaultFlags);

		rows.push_back(std::move(row));
	}

	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

void Command_Set::SaveFlagsState(const std::wstring &filePath)
{
	CTextFile tx(CP_UTF8);
	if (!tx.Open(filePath.c_str(), true))
	{
		throw EraCMDException(IDS_TXFMT_CMDERROR_FILE_OPEN_ERROR, filePath.c_str());
	}

	// UTF8 signature
	UCHAR bom[3] = {0xEF, 0xBB, 0xBF};
	if (!tx.Write(bom, sizeof(bom)))
		throw EraCMDException(CMDERROR_CANNOT_WRITE_TO_FILE, filePath.c_str());

	const FlagsRegistry::Flags &flags = gFlagsRegistry.GetSupportedFlags();
	for (FlagsRegistry::Flags::const_iterator it = flags.begin(); it != flags.end(); ++it)
	{
		std::wstring currentValue = (*it)->GetFrom(currentFlags);
		ReplaceAllSubstrings(currentValue, L"\"", L"\"\"");

		std::wostringstream ss;
		ss
			<< L"set "
			<< (*it)->GetName()
			<< L" \""
			<< currentValue
			<< L'"'
		;

		if (!tx.WriteText(ss.str()))
			throw EraCMDException(CMDERROR_CANNOT_WRITE_TO_FILE, filePath.c_str());

		if (!tx.WriteLN())
			throw EraCMDException(CMDERROR_CANNOT_WRITE_TO_FILE, filePath.c_str());
	}
}

