/**
 * @file Command_Task.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_Task.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

void Command_Task::FillTaskParams(const int argNumOffset, Era_TaskParams &params)
{

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 1))
	{
		era_TaskParams_SetName(&params, element->GetString().c_str());
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 2))
	{
		era_TaskParams_SetDescription(&params, element->GetString().c_str());
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 3))
	{
		std::wstring applyAfterStr = element->GetString();

		FILETIME applyAfter_fileTime = ::ToFileTime(applyAfterStr);

		ULARGE_INTEGER applyAfter;

		applyAfter.LowPart = applyAfter_fileTime.dwLowDateTime;
		applyAfter.HighPart = applyAfter_fileTime.dwHighDateTime;

		era_TaskParams_SetApplyAfter(&params, applyAfter.QuadPart);
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 4))
	{
		if (::ToBool(element->GetString()))
		{
			era_TaskParams_SetDeleteIfCompleted(&params);
		}
	}
}

void Command_Task::FillExcludeBools(const int argNum, bool &windows, bool &linux3, bool &linux, bool &mobile)
{
	if (std::shared_ptr<Element> element = GetArgument(argNum))
	{
		typedef std::map<std::wstring, bool *> KeywordToBool;
		KeywordToBool keywordToBool;
		keywordToBool[L"windows"] = &windows;
		keywordToBool[L"linux3"] = &linux3;
		keywordToBool[L"linux"] = &linux;
		keywordToBool[L"mobile"] = &mobile;

		std::vector<const ElementWord *> words = element->GetWords();
		for (std::vector<const ElementWord *>::const_iterator it = words.begin(); it != words.end(); ++it)
		{
			std::wstring word = (*it)->GetString();

			bool found = false;
			for (KeywordToBool::const_iterator it = keywordToBool.begin(); it != keywordToBool.end(); ++it)
			{
				const std::wstring keyword = it->first;

				if (::_wcsicmp(word.c_str(), keyword.c_str()) == 0)
				{
					*(it->second) = false;
					found = true;
					break;
				}
			}

			if (! found)
			{
				std::wostringstream ss;
				for (KeywordToBool::const_iterator it = keywordToBool.begin(); it != keywordToBool.end(); ++it)
				{
					if (it != keywordToBool.begin())
					{
						ss << L", ";
					}

					ss << it->first;
				}

				throw EraCMDException(CMDERROR_INVALID_ITEM_ONE_OF_EXPECTED, word.c_str(), ss.str().c_str());
			}
		}
	}
}

