/**
 * @file FlagsRegistry.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "FlagsRegistry.h"

#include <Common/era_api/err_codes.h>
#include "../console_colors/ConsoleColors_Win.h"
#include "../EracmdExceptions.h"
#include "FlagValuesConversions.h"

extern ConsoleColors_Win gConsoleColors;

FlagsRegistry::FlagsRegistry()
{
	flags.push_back(std::make_shared<Flag<CommandFlags::OuputFormatting> >(
		L"format",
		&CommandFlags::outputFormatting,
		&FlagValuesConversions::FromOutputFormatting,
		&FlagValuesConversions::ToOutputFormatting
	));

	flags.push_back(std::make_shared<Flag<wchar_t> >(
		L"delim",
		&CommandFlags::outputCSVDelimiter,
		&FlagValuesConversions::FromDelimiter,
		&FlagValuesConversions::ToDelimiter
	));

	flags.push_back(std::make_shared<Flag<std::wstring> >(
		L"out",
		&CommandFlags::outputFilePath,
		&FlagValuesConversions::FromString,
		&FlagValuesConversions::ToWritablePath
	));

	flags.push_back(std::make_shared<Flag<CommandFlags::OutputMode> >(
		L"mode",
		&CommandFlags::outputMode,
		&FlagValuesConversions::FromOutputMode,
		&FlagValuesConversions::ToOutputMode
	));

	flags.push_back(std::make_shared<Flag<CommandFlags::Encoding> >(
		L"enc",
		&CommandFlags::outputEncoding,
		&FlagValuesConversions::FromEncoding,
		&FlagValuesConversions::ToEncoding
	));

	flags.push_back(std::make_shared<Flag<CommandFlags::HeaderType> >(
		L"header",
		&CommandFlags::headerType,
		&FlagValuesConversions::FromHeaderType,
		&FlagValuesConversions::ToHeaderType
	));

	flags.push_back(std::make_shared<Flag<bool> >(
		L"paged",
		&CommandFlags::paged,
		&FlagValuesConversions::FromBool,
		&FlagValuesConversions::ToBool
	));

	flags.push_back(std::make_shared<Flag<bool> >(
		L"tableclip",
		&CommandFlags::tableClip,
		&FlagValuesConversions::FromBool,
		&FlagValuesConversions::ToBool
	));

	flags.push_back(std::make_shared<Flag<bool> >(
		L"color",
		&CommandFlags::color,
		&FlagValuesConversions::FromBool,
		&FlagValuesConversions::ToBool,
		std::bind(&ConsoleColors_Win::SetColoring, &gConsoleColors, std::placeholders::_1)
	));

	flags.push_back(std::make_shared<Flag<CommandFlags::FieldFormatting> >(
		L"field",
		&CommandFlags::fieldFormatting,
		&FlagValuesConversions::FromFieldFormatting,
		&FlagValuesConversions::ToFieldFormatting
	));

	flags.push_back(std::make_shared<Flag<bool> >(
		L"onerror",
		&CommandFlags::continueOnError,
		std::bind(&FlagValuesConversions::FromBoolWithCustomNames, std::placeholders::_1, L"continue", L"stop"),
		std::bind(&FlagValuesConversions::ToBoolWithCustomNames, std::placeholders::_1, L"continue", L"stop")
	));

	/*
	flags.push_back(std::make_shared<Flag<bool> >(
		L"check",
		&CommandFlags::checkOnly,
		&FlagValuesConversions::FromBool,
		&FlagValuesConversions::ToBool
	));

	flags.push_back(std::make_shared<Flag<std::wstring> >(
		L"log",
		&CommandFlags::logFilePath,
		&FlagValuesConversions::FromString,
		&FlagValuesConversions::ToWritablePath
	));

	flags.push_back(std::make_shared<Flag<CommandFlags::Encoding> >(
		L"logEnc",
		&CommandFlags::logEncoding,
		&FlagValuesConversions::FromEncoding,
		&FlagValuesConversions::ToEncoding
	));
	*/
}

const FlagsRegistry::Flags &FlagsRegistry::GetSupportedFlags() const
{
	return flags;
}

std::shared_ptr<FlagBase> FlagsRegistry::GetFlag(const std::wstring &name)
{
	for (Flags::const_iterator it = flags.begin(); it != flags.end(); ++it)
	{
		if (::_wcsicmp(name.c_str(), (*it)->GetName().c_str()) == 0)
		{
			return *it;
		}
	}

	throw EraCMDException(CMDERROR_UNSUPPORTED_FLAG, name.c_str());
}

bool FlagsRegistry::IsFlag(const std::wstring &str, const std::wstring &prefix) const
{
	for (Flags::const_iterator it = flags.begin(); it != flags.end(); ++it)
	{
		if (::_wcsicmp(str.c_str(), (prefix + (*it)->GetName()).c_str()) == 0)
		{
			return true;
		}
	}

	return false;
}

