/**
 * @file OutputFormatter_CSV.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "OutputFormatter_CSV.h"

#include "../EracmdExceptions.h"

OutputFormatter_CSV::OutputFormatter_CSV(const WCHAR fieldDelimiter, const WCHAR textDelimiter, ConsoleColors_Win &consoleColors)
:
	fieldDelimiter(fieldDelimiter),
	textDelimiter(textDelimiter),
	consoleColors(consoleColors)
{
}

void OutputFormatter_CSV::Print(TableReader &reader, std::wostream &out, const CommandFlags::HeaderType headerType, DWORD screenWidth, const TextWidth &textWidth)
{

	out << consoleColors.AQUA;
	if (headerType == CommandFlags::HEADER_KEYWORD)
	{
		PrintRow(reader.GetColumns_keyword(), out);
	}
	else if (headerType == CommandFlags::HEADER_PRETTY)
	{
		PrintRow(reader.GetColumns_pretty(), out);
	}

	int lineIndex = 1;
	while (reader.HasNext())
	{
		const StringVector &row = reader.GetNext();

		if (lineIndex % 2 == 0)
			out << consoleColors.LIGHT_AQUA;
		else
			out << consoleColors.DEFAULT;

		PrintRow(row, out);
		lineIndex++;
	}
	out << consoleColors.DEFAULT;
}

void OutputFormatter_CSV::PrintRow(const StringVector &row, std::wostream &out)
{
	for (StringVector::const_iterator it = row.begin(); it != row.end(); ++it)
	{
		if (it != row.begin())
		{
			out << fieldDelimiter;
		}

		const std::wstring &fieldText = *it;

		bool needsTextDelimiter = (
			(fieldText.find(fieldDelimiter) != std::wstring::npos)
			|| (fieldText.find(textDelimiter) != std::wstring::npos)
			|| (fieldText.find_first_of(L"\r\n") != std::wstring::npos)
		);

		if (needsTextDelimiter)
		{
			out << textDelimiter;

			std::wstring::size_type begin = 0;
			do
			{
				std::wstring::size_type textDelimPos = fieldText.find(textDelimiter, begin);
				if (textDelimPos == std::wstring::npos)
				{
					out << fieldText.substr(begin);
					break;
				}
				else
				{
					out << fieldText.substr(begin, textDelimPos - begin);

					out << textDelimiter << textDelimiter;

					begin = textDelimPos + 1;
				}
			} while (true);

			out << textDelimiter;
		}
		else
		{
			out << fieldText;
		}
	}
	out << std::endl;
}

